/*
 * Copyright 2011, The gwtquery team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.google.gwt.query.client.css;

import com.google.gwt.dom.client.Style.TextDecoration;

/**
 * The <i>text-decoration</i> property describes decorations that are added to
 * the text of an element using the element's color. When specified on or
 * propagated to an inline element, it affects all the boxes generated by that
 * element, and is further propagated to any in-flow block-level boxes that
 * split the inline. For block containers that establish an inline formatting
 * context, the decorations are propagated to an anonymous inline element that
 * wraps all the in-flow inline-level children of the block container. For all
 * other elements it is propagated to any in-flow children. Note that text
 * decorations are not propagated to floating and absolutely positioned
 * descendants, nor to the contents of atomic inline-level descendants such as
 * inline blocks and inline tables.
 */
public class TextDecorationProperty extends CssProperty<TextDecoration> {

  private static final String CSS_PROPERTY = "textDecoration";

  public static void init() {
    CSS.TEXT_DECORATION = new TextDecorationProperty();
  }

  private TextDecorationProperty() {
    super(CSS_PROPERTY);
  }
}
